function varargout = zephyr(varargin)

% ZEPHYR SPM2 Batch Processing Scripts
%
% By Craig M. Bennett
% The Laboratory for Adolescent Studies
% Dartmouth College
%
% 6207 Moore Hall
% Hanover, NH 03755
% cbspam@dartmouth.edu
%
% Version .3 - Beta Software
% Copyright 2004, Craig M. Bennett
%
% --------------------------------------------------------------
%
% Some code snippets from SPM2 used under the GNU GPL
% Copyright 2004 Functional Imaging Laboratory (FIL) 
% The Wellcome Department of Imaging Neuroscience
% Institute of Neurology at University College London
%
% --------------------------------------------------------------
%
% Zephyr is free software; you can redistribute it and/or modify
% it under the terms of the GNU General Public License as published by
% the Free Software Foundation; either version 2 of the License, or
% (at your option) any later version.
%
% Zephyr is distributed in the hope that it will be useful,
% but WITHOUT ANY WARRANTY; without even the implied warranty of
% MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
% GNU General Public License for more details.
%
% You should have received a copy of the GNU General Public License
% along with Zephyr (zephyr_license.txt); if not, write to the Free Software
% Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
%
%
% --------------------------------------------------------------
%
%
% zephyr.m


% Begin initialization code - DO NOT EDIT
    gui_Singleton = 1;
    gui_State = struct('gui_Name',       mfilename, ...
                       'gui_Singleton',  gui_Singleton, ...
                       'gui_OpeningFcn', @zephyr_OpeningFcn, ...
                       'gui_OutputFcn',  @zephyr_OutputFcn, ...
                       'gui_LayoutFcn',  [] , ...
                       'gui_Callback',   []);
    if nargin & isstr(varargin{1})
        gui_State.gui_Callback = str2func(varargin{1});
    end

    if nargout
        [varargout{1:nargout}] = gui_mainfcn(gui_State, varargin{:});
    else
        gui_mainfcn(gui_State, varargin{:});
    end
% End initialization code - DO NOT EDIT



% --- Executes just before zephyr is made visible.
function zephyr_OpeningFcn(hObject, eventdata, handles, varargin)

    global zephyr_struct;
    
    zephyr_struct_initialize;
    
    % Initialize the GUI
    zephyr_gui_manage;
    
    % Choose default command line output for zephyr
    handles.output = hObject;

    % Update handles structure
    guidata(hObject, handles);
    
    % DBIC Problem Solver
	current_path = pwd;
	if (strcmp(current_path(1:23),'/afs/dbic.dartmouth.edu') == 1)
		addpath /afs/dbic.dartmouth.edu/usr/pkg/GE2SPM/GE2SPM_v3.3
		addpath /afs/dbic.dartmouth.edu/usr/pkg/spm/spm2
		addpath /afs/dbic.dartmouth.edu/usr/pkg/zephyr
	end


    
% --- Outputs from this function are returned to the command line.
function varargout = zephyr_OutputFcn(hObject, eventdata, handles)

    % Get default command line output from handles structure
    varargout{1} = handles.output;


    
% --- Executes on button press in directory_select.
function directory_select_Callback(hObject, eventdata, handles)
    
    zephyr_struct_initialize;
    
    global zephyr_struct;
    
    % let the user find the data directory
    zephyr_struct.data_directory = spm_get(-1,'','Select directory of subject folders','',0);
    cd(zephyr_struct.data_directory);
    
    disp(' '); disp(['Current data directory: ' zephyr_struct.data_directory]); disp(' ');
    
    % start from scratch or continue where we left off
    if (exist('./zephyr/zephyr_data.mat') < 1)
        
		unix('mkdir zephyr');
        disp(' '); disp('Zephyr directory created'); disp(' ');

        zephyr_get_folders
        
        % Put a copy of the defaults in the Zephyr folder for the user to modify
        zephyr_path = which('zephyr.m');
        zephyr_path = fileparts(zephyr_path);
        unix(['cp ' zephyr_path '/zephyr_preprocessing_defaults.m ' zephyr_struct.data_directory '/zephyr/']); 
        unix(['cp ' zephyr_path '/zephyr_postprocessing_defaults.m ' zephyr_struct.data_directory '/zephyr/']);
        
        zephyr_struct.processing_step = 5;
        save('./zephyr/zephyr_data.mat','zephyr_struct')
        
        zephyr_gui_manage;
        
    else
        load('./zephyr/zephyr_data.mat');
        disp(' '); disp('Study information loaded'); disp(' ');
        
        zephyr_update_folders;
        
        zephyr_gui_manage;
        
	end		
    
    % Update handles structure
    guidata(hObject, handles);

    
% --- Executes on button press in input_study.
function input_study_Callback(hObject, eventdata, handles)

    global zephyr_struct;

    % get the overall study parameters
    fig = zephyr_study_parameters;
    uiwait(fig);
    
    % get the individual scan information
    for i = 1:zephyr_struct.num_scans
        fig = zephyr_scan_parameters(i);
        uiwait(fig);
    end
    
    % Update the gui, progress has been made
    zephyr_struct.processing_step = 15;
    save('./zephyr/zephyr_data.mat','zephyr_struct')
    zephyr_gui_manage;
    

% --- Executes on button press in preprocess_options.
function preprocess_options_Callback(hObject, eventdata, handles)



% --- Executes on button press in select_models.
function select_models_Callback(hObject, eventdata, handles)

    global zephyr_struct;
    
    model_count = 0;
    
    cd(zephyr_struct.data_directory);
    
    % Get the models for each functional series and specify the data
    for j=1:zephyr_struct.num_scans
        if (strcmp(char(zephyr_struct.study_metadata(j,1)),'Functional') == 1)
            model_location = spm_get([1,1],'SPM.mat',['Select ' char(zephyr_struct.study_metadata(j,3)) ' model'],'',0);
            unix(['cp ' model_location ' ./results/' num2str(j) '-' char(zephyr_struct.study_metadata(j,3))]);
            model_count = model_count + 1;
            
        end
    end

    helpdlg([num2str(model_count) ' Models Copied'],'Model Acquisition');
    
    load([zephyr_struct.data_directory '/zephyr/zephyr_data.mat']);
        zephyr_struct.processing_step = 25;
        zephyr_gui_manage
    save([zephyr_struct.data_directory '/zephyr/zephyr_data.mat'],'zephyr_struct')


% --- Executes on button press in estimate.
function estimate_Callback(hObject, eventdata, handles)

    set(findobj('Tag','alert_text'),'String','Postprocessing');
    set(findobj('Tag','alert_text'),'BackgroundColor',[.64 .021 .0]);
    set(findobj('Tag','alert_frame'),'BackgroundColor',[.64 .021 .0]);
    drawnow;
    
    fig = zephyr_postprocess;
    uiwait(fig);

    zephyr_gui_manage;

% --- Executes on button press in grid_participate.
function grid_participate_Callback(hObject, eventdata, handles)



% --- Executes on button press in change_defaults.
function change_defaults_Callback(hObject, eventdata, handles)



% --- Executes on button press in preprocess.
function preprocess_Callback(hObject, eventdata, handles)

    global zephyr_struct;    

    set(findobj('Tag','alert_text'),'String','Preprocessing');
    set(findobj('Tag','alert_text'),'BackgroundColor',[.64 .021 .0]);
    set(findobj('Tag','alert_frame'),'BackgroundColor',[.64 .021 .0]);
    drawnow;
    
    fig = zephyr_preprocess;
    uiwait(fig);

    load([zephyr_struct.data_directory '/zephyr/zephyr_data.mat']);
    zephyr_struct.processing_step = 20;
    zephyr_gui_manage
    save([zephyr_struct.data_directory '/zephyr/zephyr_data.mat'],'zephyr_struct')

% --- Executes on button press in test_data.
function test_data_Callback(hObject, eventdata, handles)

zephyr_test











% --- Function to manage the GUI buttons and such
function zephyr_gui_manage()

    global zephyr_struct;

    set(findobj('Tag','alert_text'),'String','Copyright 2004, Version .3B');
    set(findobj('Tag','alert_text'),'BackgroundColor',[.0 .006 .36]);
    set(findobj('Tag','alert_frame'),'BackgroundColor',[.0 .006 .36]);
    
    
    if (zephyr_struct.processing_step == 0)
        
        % turn off the user interface bits we can't use yet
        set(findobj('Tag','input_study'),'Enable','off');
        
        set(findobj('Tag','test_data'),'Enable','off');

        set(findobj('Tag','preprocess'),'Enable','off');
        
        set(findobj('Tag','select_models'),'Enable','off');
        
        set(findobj('Tag','estimate'),'Enable','off');
                
    elseif (zephyr_struct.processing_step == 5)
    
        % turn on newly available buttons
        set(findobj('Tag','input_study'),'Enable','on');
    
    elseif (zephyr_struct.processing_step == 15)
        
        % from previous steps
        set(findobj('Tag','input_study'),'Enable','on');
        
        % preprocessing is available
        set(findobj('Tag','test_data'),'Enable','on');
        set(findobj('Tag','preprocess'),'Enable','on');
        
    elseif (zephyr_struct.processing_step == 20)
        
        % from previous steps
        set(findobj('Tag','input_study'),'Enable','on');
        set(findobj('Tag','test_data'),'Enable','on');
        set(findobj('Tag','preprocess'),'Enable','on');
        
        % preprocessing is done, need info on the design
        set(findobj('Tag','select_models'),'Enable','on');
    
    elseif (zephyr_struct.processing_step == 25)
        
        % from previous steps
        set(findobj('Tag','input_study'),'Enable','on');
        set(findobj('Tag','test_data'),'Enable','on');
        set(findobj('Tag','preprocess'),'Enable','on');
        set(findobj('Tag','select_models'),'Enable','on');
        
        % Ready to finish...
        set(findobj('Tag','estimate'),'Enable','on');
        
    end
    
    drawnow;
    
    
    
    
% --- Function to find data folders
function zephyr_get_folders()

    global zephyr_struct;
    
    set(findobj('Tag','alert_text'),'String','Adding & Decompressing Data');
    set(findobj('Tag','alert_text'),'BackgroundColor',[.64 .021 .0]);
    set(findobj('Tag','alert_frame'),'BackgroundColor',[.64 .021 .0]);
    drawnow;
    
    cd(zephyr_struct.data_directory);
    
    % get list of all data folders and put in an array
    % this is a hack, but it works, so pbbbbbtttt....
    unix('for fn in *; do echo "$fn" >> ./zephyr/subject_folder_list.txt; done;');
    pause(2);
    [file_names] = textread ('./zephyr/subject_folder_list.txt','%s');
    unix('rm ./zephyr/subject_folder_list.txt');
    
    num_folders = length(file_names);

    for k=1:num_folders
	    if ((strcmp(char(file_names(k)),'.') == 1) | (strcmp(char(file_names(k)),'..') == 1) | (strcmp(char(file_names(k)),'zephyr') == 1) | (strcmp(char(file_names(k)),'results') == 1) | (strcmp(char(file_names(k)),'models') == 1))
                % this block is for folders that are not subjects
                % really it does nothing right now and needs to do nothing
        else
		        % this block means the folder contains a subject
		        % add it to the array and increment the counter
                
                % if it is in a compressed archive then decompress the info
                [testpath,testname,testext] = fileparts(char(file_names(k)));
                if (strcmp(testext,'.tgz'))
                    disp(' ');
                    disp(['Decompressing ' char(file_names(k))]);
                    unix(['gunzip ' char(file_names(k))]);
                    disp(['Unpacking ' testname '.tar']);
                    unix(['tar -xvf ' testname '.tar']);
                    unix(['rm ' testname '.tar']);
                    file_names(k) = {testname};
                end
                
		        zephyr_struct.num_subjects = zephyr_struct.num_subjects + 1;
		        zephyr_struct.processing_track(zephyr_struct.num_subjects,1) = file_names(k);
                zephyr_struct.processing_track(zephyr_struct.num_subjects,2:13) = {'-'};
	    end
        
    end

	helpdlg([num2str(zephyr_struct.num_subjects) ' Data Folders Found'],'Data Acquisition');
    
    save('./zephyr/zephyr_data.mat','zephyr_struct')
    
    
    
    
    
% --- Function to find new data folders
function zephyr_update_folders()

    global zephyr_struct;
    
    cd(zephyr_struct.data_directory);
    
    % get list of all data folders and put in an array
    % this is a hack, but it works, so pbbbbbtttt....
    unix('for fn in *; do echo "$fn" >> ./zephyr/subject_folder_list.txt; done;');
    [file_names] = textread ('./zephyr/subject_folder_list.txt','%s');
    unix('rm ./zephyr/subject_folder_list.txt');
    
    num_folders = length(file_names);
    num_subjects = 0;
    
    
    for k=1:num_folders
	    if ((strcmp(char(file_names(k)),'.') == 1) | (strcmp(char(file_names(k)),'..') == 1) | (strcmp(char(file_names(k)),'zephyr') == 1) | (strcmp(char(file_names(k)),'results') == 1) | (strcmp(char(file_names(k)),'models') == 1))
                % this block is for folders that are not subjects
                % really it does nothing right now and needs to do nothing
        else
		        % this block means the folder contains a subject
		        % add it to the array and increment the counter
                
                % if it is in a compressed archive then decompress the info
                [testpath,testname,testext] = fileparts(char(file_names(k)));
                if (strcmp(testext,'.tgz'))
                    disp(' ');
                    disp(['Decompressing ' char(file_names(k))]);
                    unix(['gunzip ' char(file_names(k))]);
                    disp(['Unpacking ' testname '.tar']);
                    unix(['tar -xvf ' testname '.tar']);
                    unix(['rm ' testname '.tar']);
                    file_names(k) = {testname};
                end
                
		        num_subjects = num_subjects + 1;
                subject_list(num_subjects) = file_names(k);
	    end
    end
    
    
    if (num_subjects ~= zephyr_struct.num_subjects)
            
        % Give the user the options
        num_new = num_subjects - zephyr_struct.num_subjects;
        choice = questdlg([num2str(num_new) ' new data folders found.  Add them for processing?'],'Data Acquisition','Add','Ignore','Add');
            
        if (strcmp(choice,'Add') == 1)
            	for t = 1 : num_subjects
		            exists = 0;

		            for s = 1 : zephyr_struct.num_subjects
			            if (strcmp(char(subject_list(t)),char(zephyr_struct.processing_track(s,1))) == 1)
				            exists = 1;
			            end
		            end
	
		            if (exists == 0)
			            zephyr_struct.num_subjects =  zephyr_struct.num_subjects + 1;
                        zephyr_struct.processing_track(zephyr_struct.num_subjects,1) = subject_list(t);
			            zephyr_struct.processing_track(zephyr_struct.num_subjects,2:13) = {'-'};
		            end

	            end
    
                save('./zephyr/zephyr_data.mat','zephyr_struct')
        end
            

    end
        
    
% --- Function to initialize the main zephyr_struct
function zephyr_struct_initialize()    
    
    clear global zephyr_struct;

    global zephyr_struct;
    
    % initialize the zephyr structure
    zephyr_struct.study_name = '';
    zephyr_struct.data_directory = '';
    zephyr_struct.processing_step = 0;
    zephyr_struct.num_scans = 0;
    zephyr_struct.num_subjects = 0;
    zephyr_struct.scan_types = struct('scout',0,'coplanar',0,'functional',0,'hires',0,'dti',0);
    zephyr_struct.study_metadata(1,1:6) = {''};
    zephyr_struct.processing_track(1,1:13) = {''};
    zephyr_struct.conditions_metadata = {''};    
    
    
    
    
    
    

    