% ZEPHYR_COREGISTER
%
% Version .3 - Beta Software
% Copyright 2004, Craig M. Bennett
%
% --------------------------------------------------------------
%
% Some code snippets from SPM2 used under the GNU GPL
% Copyright 2004 Functional Imaging Laboratory (FIL) 
% The Wellcome Department of Imaging Neuroscience
% Institute of Neurology at University College London
%
% --------------------------------------------------------------
%
% This file is part of Zephyr.
%
% Zephyr is free software; you can redistribute it and/or modify
% it under the terms of the GNU General Public License as published by
% the Free Software Foundation; either version 2 of the License, or
% (at your option) any later version.
%
% Zephyr is distributed in the hope that it will be useful,
% but WITHOUT ANY WARRANTY; without even the implied warranty of
% MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
% GNU General Public License for more details.
%
% You should have received a copy of the GNU General Public License
% along with Zephyr (zephyr_license.txt); if not, write to the Free Software
% Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
%
% --------------------------------------------------------------
%
% zephyr_coregister.m


function zephyr_coregister()

	global zephyr_struct;
	global defaults;
	
	defaults.cmdline = 0;
    defaults.analyze.flip = 1;
    defaults.printstr =  [spm_figure('DefPrintCmd'),'spm2.ps'];

    % Get processing parameters from the user's defaults file
    run ../zephyr/zephyr_preprocessing_defaults.m
    
    %% Coregister Hires & Coplanar %%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
    
    disp(' '); disp('Coregistering Hires & Coplanar...');
      
    if ((zephyr_struct.scan_types.coplanar == 1) & (zephyr_struct.scan_types.hires == 1))
    
        target_images = '';
        
        % find the first hires file
        for j=1:zephyr_struct.num_scans
            if (strcmp(char(zephyr_struct.study_metadata(j,1)),'Hires') == 1) 
                hires_path = ['./' num2str(j) '-Hires'];    
                hires = spm_get('files',hires_path,'hires.img');
                hires_vol = spm_vol(hires);
                break;
            end
        end
        
        % find the first coplanar file
        for j=1:zephyr_struct.num_scans
           if (strcmp(char(zephyr_struct.study_metadata(j,1)),'Coplanar') == 1) 
              coplanar_path = ['./' num2str(j) '-Coplanar'];    
              coplanar = spm_get('files',coplanar_path,'coplanar.img');
              target_images = coplanar;
              coplanar_vol = spm_vol(coplanar);
              break;
           end
        end
       
   
        % drag the functional images along, if present
        if ((zephyr_struct.scan_types.functional == 1))
            for j=1:zephyr_struct.num_scans
                if (strcmp(char(zephyr_struct.study_metadata(j,1)),'Functional') == 1)  
                    
                    if (strcmp(char(zephyr_struct.study_metadata(j,6)),'Event') == 1)
				        prefix = 'ua';
			        elseif (strcmp(char(zephyr_struct.study_metadata(j,6)),'Block') == 1)
				        prefix = 'u';
			        end
                   
                    scan_dir = ['./' num2str(j) '-' char(zephyr_struct.study_metadata(j,3))];
                    
                    % throw the mean EPI image in there...
                    mean_image = spm_get('files',scan_dir,['mean' prefix char(zephyr_struct.study_metadata(j,3)) '*.img']);

                    % and the rest of 'em...
                    image_matrix = spm_get('files',scan_dir,[prefix char(zephyr_struct.study_metadata(j,3)) '*.img']);
                       
                    target_images = char(target_images,mean_image,image_matrix);
                    
                end
            end
        end        
        
        
        % coregister
        x = spm_coreg(hires_vol,coplanar_vol,first_coregister_flags);
        
        
        % borrowed SPM2 spm_coreg_ui.m code (no clue what it is doing)
        M  = inv(spm_matrix(x));
        
		MM = zeros(4,4,size(target_images,1));
        
		for j=1:size(target_images,1),
			MM(:,:,j) = spm_get_space(deblank(target_images(j,:)));
		end;
        
		for j=1:size(target_images,1),
			spm_get_space(deblank(target_images(j,:)), M*MM(:,:,j));
		end;
       

    end
    
    
    
    
    %% Coregister Hires & Mean EPI %%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
    
    disp(' '); disp('Coregistering Hires & Mean EPI...');
    
    if ((zephyr_struct.scan_types.functional == 1) & (zephyr_struct.scan_types.hires == 1))
    
        % find the first hires file
        for j=1:zephyr_struct.num_scans
            if (strcmp(char(zephyr_struct.study_metadata(j,1)),'Hires') == 1) 
                hires_path = ['./' num2str(j) '-Hires'];    
                hires = spm_get('files',hires_path,'hires.img');
                hires_vol = spm_vol(hires);
                break;
            end
        end
        
       % do a second coregistration on each functional run 
       for j=1:zephyr_struct.num_scans
            if (strcmp(char(zephyr_struct.study_metadata(j,1)),'Functional') == 1)
                
                    target_images = '';
                
                    if (strcmp(char(zephyr_struct.study_metadata(j,6)),'Event') == 1)
				        prefix = 'ua';
			        elseif (strcmp(char(zephyr_struct.study_metadata(j,6)),'Block') == 1)
				        prefix = 'u';
			        end
        
                    scan_dir = ['./' num2str(j) '-' char(zephyr_struct.study_metadata(j,3))];
                    
                    % get the mean image and the rest of the EPI images
                    mean_image = spm_get('files',scan_dir,['mean' prefix char(zephyr_struct.study_metadata(j,3)) '*.img']);
                    image_matrix = spm_get('files',scan_dir,[prefix char(zephyr_struct.study_metadata(j,3)) '*.img']);
                    
                    % put the images in the target array
                    target_images = char(mean_image,image_matrix);
                    
                    % get the mean volume ready for passing
                    mean_vol = spm_vol(mean_image);
                    
                    % coregister!
                    x = spm_coreg(hires_vol,mean_vol,second_coregister_flags);
            
                    % borrowed SPM2 spm_coreg_ui.m code (no clue what it is doing)
                    M  = inv(spm_matrix(x));
        
		            MM = zeros(4,4,size(target_images,1));
        
		            for j=1:size(target_images,1),
			            MM(:,:,j) = spm_get_space(deblank(target_images(j,:)));
		            end;
        
		            for j=1:size(target_images,1),
			            spm_get_space(deblank(target_images(j,:)), M*MM(:,:,j));
		            end
                    
            end
       end    
           
   end  
   
   disp(' '); disp('Coregistration Complete');disp('');
