% ZEPHYR_DTI_COMPUTE
%
% Version .3 - Beta Software
% Copyright 2004, Craig M. Bennett
%
% --------------------------------------------------------------
%
% Some code snippets from SPM2 used under the GNU GPL
% Copyright 2004 Functional Imaging Laboratory (FIL) 
% The Wellcome Department of Imaging Neuroscience
% Institute of Neurology at University College London
%
% --------------------------------------------------------------
%
% This file is part of Zephyr.
%
% Zephyr is free software; you can redistribute it and/or modify
% it under the terms of the GNU General Public License as published by
% the Free Software Foundation; either version 2 of the License, or
% (at your option) any later version.
%
% Zephyr is distributed in the hope that it will be useful,
% but WITHOUT ANY WARRANTY; without even the implied warranty of
% MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
% GNU General Public License for more details.
%
% You should have received a copy of the GNU General Public License
% along with Zephyr (zephyr_license.txt); if not, write to the Free Software
% Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
%
% --------------------------------------------------------------
%
% zephyr_dti_compute.m

function zephyr_slice_timing()

	global zephyr_struct;
	global defaults;
	
	defaults.cmdline = 0;
    defaults.analyze.flip = 1;
    defaults.printstr =  [spm_figure('DefPrintCmd'),'spm2.ps'];

    % Get processing parameters from the user's defaults file
    run ../zephyr/zephyr_preprocessing_defaults.m
    
	disp(' '); disp('Computing DTI Images: Creating Brainmask');

	% find the normalized hires to segment
	for j=1:zephyr_struct.num_scans
		if (strcmp(char(zephyr_struct.study_metadata(j,1)),'Hires') == 1) 
		
			cd(['./' num2str(j) '-Hires']);;  
                 
			% segment the whires image and create a brain mask
			%spm_segment('./whires.img',eye(4),segment_flags);
			
			spm_imcalc_ui(['./whires_seg1.img'; './whires_seg2.img'],['./mask'],['i1 + i2']);
			spm_smooth(['./mask.img'],['./smask'],1);
			spm_imcalc_ui(['./smask.img'],['./brainmask'],['i1 > .1']);
				
			mask_path = pwd;
			unix(['rm smask.img mask.img']);

			break;
        
		end
	end



    for j=1:zephyr_struct.num_scans
        if (strcmp(char(zephyr_struct.study_metadata(j,1)),'DTI') == 1)    

			cd(['../' num2str(j) '-DTI']);

			% rename the images for the DTI script
			for b = 1:7
				unix(['mv dti_i000' num2str(b) '.hdr b' num2str(b-1) '.hdr']);
				unix(['mv dti_i000' num2str(b) '.img b' num2str(b-1) '.img']);
				unix(['mv dti_i000' num2str(b) '.mat b' num2str(b-1) '.mat']);
				
			end

			% get the brainmask
			unix(['cp ' mask_path '/brainmask.img ./']);
			
			disp(' '); disp('Computing DTI Images: Creating Images');disp(' ');
			
			%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
			%% Begin DBIC DTI computation script
			
			% Initialize stuff	
			bvalue = 1000;
			maskthresh = .2; 	% The minimum mask value for inclusion (ie chance
                 				% of being brain as defined by the spm segmentation)
		 
			% The working directory
			workdir = pwd; 
			bdir = workdir;
			adcdir = workdir;

			% Set the number of bimages
			numb = 7;

			% Create the structures for the input files
			for i = 1:numb
  				P(i,:) = fullfile(bdir,sprintf('b%d.img',i-1));
			end
			Vi = spm_vol(P);
			Vi(numb+1) = spm_vol(fullfile(bdir,'brainmask.img'));

			% Check for the existence of the output directory.
			% If it's not there make it
			[adcdirroot adcdirname] = fileparts(adcdir);
			status = mkdir(adcdirroot,adcdirname);
			if ( status == 1 )
				fprintf('Created adc directory:\n%s\n', adcdir);
			elseif ( status == 2 )
				fprintf('Warning adc directory exists:\n%s\nOverwriting...\n', adcdir);
			else
				error(sprintf('Error creating adc directory: %s\n', adcdir))
			end

			% Create the structures for the output images
			Vo(1) = Vi(1); % Initialize the first one
			Vo(1).dim(4) = spm_type('float');  % Floating point images
			count = 1;     % Set the counter

			for i = 1:6
				Vo(count) = Vo(1);
				Vo(count).fname = fullfile(adcdir,sprintf('adc%d.img',i));
				Vo(count).descrip = 'Raw ADC';
				Vo(count) = spm_create_vol(Vo(count));
				count = count +1;
			end

			for i = 1:6
				Vo(count) = Vo(1);
				Vo(count).fname = fullfile(adcdir,sprintf('tadc%d.img',i));
				Vo(count).descrip = 'ADC in the true frame';
				Vo(count) = spm_create_vol(Vo(count));
				count = count +1;
			end

			for i = 1:3
				Vo(count) = Vo(1);
				Vo(count).fname = fullfile(adcdir,sprintf('eigval%d.img',i));
				Vo(count).descrip = 'Eigenvalue of the Tensor';
				Vo(count) = spm_create_vol(Vo(count));
				count = count +1;
			end

			Vo(count) = Vo(1);
			Vo(count).fname = fullfile(adcdir,'md.img');
			Vo(count).descrip = 'Mean Diffusivity Image';
			Vo(count) = spm_create_vol(Vo(count));
			count = count+1;

			for i = 1:3
				for j = 1:3
					Vo(count) = Vo(1);
					Vo(count).fname = fullfile(adcdir,sprintf('eigvec%d_%d.img',i,j));
					Vo(count).descrip = 'Eigenvector of the Tensor';
					Vo(count) = spm_create_vol(Vo(count));
					count = count +1;
					Vo(count) = Vo(1);
					Vo(count).fname = fullfile(adcdir,sprintf('abseigvec%d_%d.img',i,j));
					Vo(count).descrip = 'Absolute Value of the Eigenvector';
					Vo(count) = spm_create_vol(Vo(count));
					count = count +1;
				end
			end

			Vo(count) = Vo(1);
			Vo(count).fname = fullfile(adcdir,'fa.img');
			Vo(count).descrip = 'Fractional Anisotropy Image';
			Vo(count) = spm_create_vol(Vo(count));

			% The matrix relating rawadc to tadc - see the note at the end
			M = [ ...
				1  0  1  0  2  0; ...
				1  0  1  0 -2  0; ...
				0  1  1  0  0  2; ...
				0  1  1  0  0 -2; ...    
				1  1  0  2  0  0; ...
				1  1  0 -2  0  0  ...
				];
			MINV = M^(-1);

			% Loop over all the slices computing the various pieces.
			% Nearest neighbor sampling because everything is in register
			hold = 0; 
	
			% Initialize the array to load the data into
			sliceshape = [Vo(1).dim(1) Vo(1).dim(2)];
			slicesize = prod(sliceshape);
			bslice = zeros(numb,slicesize);
	
			fprintf('Percent Complete = 00');
	
			for p = 1:Vo(1).dim(3)
  				fprintf('\b\b%02d',floor(100*p/Vo(1).dim(3)));
  	
  				B = spm_matrix([0 0 -p 0 0 0 1 1 1]);
  	
  				% Loop over the b images loading them in
  				for i = 1:numb
					M = inv(B);
					bslice(i,:) = reshape( spm_slice_vol(Vi(i),M,sliceshape,[hold,NaN]), [1 slicesize] );
				end
  	
				% Load in the mask
				bslice(numb+1,:) = reshape( spm_slice_vol(Vi(numb+1),M,sliceshape,[hold,NaN]), [1 slicesize] );
  	
  				% Compute the useable voxels.  Those are inside the brain.
				%vox1 = find(prod(bslice(2:6,:),1));
				%vox2 = find(bslice(numb+1,:)>maskthresh);
				%vox = intersect(vox1,vox2);
				vox = find(bslice(numb+1,:)>maskthresh);
  	
				% Compute the raw adc images
				startim = 0;  % Initialize the output counter
				rawadc = zeros(6,slicesize);
				for i=1:6
					rawadc(i,vox) = -(1/bvalue)*log((bslice(i+1,vox)+eps)./(bslice(1,vox)+eps));
					spm_write_plane(Vo(i+startim),reshape(rawadc(i,:),sliceshape),p);
				end
			
				startim = startim + 6;
  	
				% Compute the true adc images
				tadc = zeros(6,slicesize);
				tadc(:,vox) = MINV*rawadc(:,vox);
				for i=1:6
					spm_write_plane(Vo(i+startim), reshape(tadc(i,:),sliceshape), p);
				end
			
				startim = startim + 6;
	
				% Stuff the tensor with ADC values
				tensors = zeros(3,3,slicesize);
				tensors(1,1,:) = tadc(1,:);
				tensors(2,2,:) = tadc(2,:);
				tensors(3,3,:) = tadc(3,:);
				tensors(1,2,:) = tadc(4,:);
				tensors(1,3,:) = tadc(5,:);
				tensors(2,3,:) = tadc(6,:);
				tensors(2,1,:) = tensors(1,2,:);
				tensors(3,1,:) = tensors(1,3,:);
				tensors(3,2,:) = tensors(2,3,:);
	
				% Diagonalize the tensor and write out the images
				% Perform SVD at every voxel
				ut = zeros(3,3,slicesize);
				st = zeros(3,slicesize);
				for j = 1:prod(size(vox))
					temp(:,:) = tensors(:,:,vox(j));
					[u,s,v] = svd(temp);
					ut(:,:,vox(j)) = u;
					st(:,vox(j)) = diag(s);
				end
		
				% Write out the eigenvalues
				for i=1:3
					spm_write_plane(Vo(i+startim), reshape(st(i,:),sliceshape), p);
				end
				
				startim = startim+3;
  		
				% Write out the mean diffusivity
				spm_write_plane(Vo(1+startim), reshape(sum(st,1),sliceshape), p);
				
				startim = startim+1;
  		
				% write out the eigenvectors and abs eigvec
				for i=1:3
					for j=1:3
						imcount = 2*((i-1)*3+j)-1;
						spm_write_plane(Vo(imcount+startim), ...
						reshape(ut(i,j,:),sliceshape), p);
						spm_write_plane(Vo(imcount+1+startim), ...
						reshape(abs(ut(i,j,:)),sliceshape), p);
					end
				end
				
				startim = startim + 18;
		
				% Compute the fractional anisotropy
				fa = zeros(1,slicesize);
				for j = 1:prod(size(vox))
					lam = st(:,vox(j));
					l = mean(lam);
					if (lam'*lam > eps)
						fa(vox(j)) = sqrt(3/2*((lam-l)'*(lam-l))/(lam'*lam));
					end
				end
				spm_write_plane(Vo(startim+1), reshape(fa,sliceshape), p);
		
			end % slice loop
	
			%% End DBIC DTI computation script
			%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%


			cd(['..']);

        end
    end
    
    disp(' '); disp('DTI Images Complete'); disp(' ');