% ZEPHYR_NORMALISE
%
% Version .3 - Beta Software
% Copyright 2004, Craig M. Bennett
%
% --------------------------------------------------------------
%
% Some code snippets from SPM2 used under the GNU GPL
% Copyright 2004 Functional Imaging Laboratory (FIL) 
% The Wellcome Department of Imaging Neuroscience
% Institute of Neurology at University College London
%
% --------------------------------------------------------------
%
% This file is part of Zephyr.
%
% Zephyr is free software; you can redistribute it and/or modify
% it under the terms of the GNU General Public License as published by
% the Free Software Foundation; either version 2 of the License, or
% (at your option) any later version.
%
% Zephyr is distributed in the hope that it will be useful,
% but WITHOUT ANY WARRANTY; without even the implied warranty of
% MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
% GNU General Public License for more details.
%
% You should have received a copy of the GNU General Public License
% along with Zephyr (zephyr_license.txt); if not, write to the Free Software
% Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
%
% --------------------------------------------------------------
%
% zephyr_normalise.m


function zephyr_normalise()

	global zephyr_struct;
	global defaults;
	
	defaults.cmdline = 0;
    defaults.analyze.flip = 1;
    defaults.printstr =  [spm_figure('DefPrintCmd'),'spm2.ps'];
    
    % Get processing parameters from the user's defaults file
    run ../zephyr/zephyr_preprocessing_defaults.m
    
%%% Determine parameters from the first Hires image %%%%%%%%%%%%%%%

    disp(' '); disp('Determining normalisation parameters...'); disp(' ');

    if (zephyr_struct.scan_types.hires == 1)

        % Normalise the first hires image
        for j=1:zephyr_struct.num_scans
            if (strcmp(char(zephyr_struct.study_metadata(j,1)),'Hires') == 1) 
        
                template = [spm('Dir') '/templates/T1.mnc'];
                
                hires_path = ['./' num2str(j) '-Hires'];    
                hires_image = spm_get('files',hires_path,'hires.img');

                matname = [hires_path '/hires_sn3d.mat'];
                
                parameters = spm_normalise(template,hires_image,matname,'','',normalise_estimate_flags);

                break;
        
            end
        end
 
    end
    
    
%%% Apply parameters to all Hires & EPI images %%%%%%%%%%%%%%%
    
    disp(' '); disp('Applying normalisation parameters...'); disp(' ');

    for j=1:zephyr_struct.num_scans
        if (strcmp(char(zephyr_struct.study_metadata(j,1)),'Hires') == 1)
        
            hires_path = ['./' num2str(j) '-Hires']; 
            disp(['Normalising: ' num2str(j) '-Hires']);
            hires_image = spm_get('files',hires_path,'hires.img');
        
            spm_write_sn(hires_image,parameters,normalise_hires_flags)
        
        elseif (strcmp(char(zephyr_struct.study_metadata(j,1)),'Functional') == 1)
                
            target_images = '';
                
            if (strcmp(char(zephyr_struct.study_metadata(j,6)),'Event') == 1)
                prefix = 'ua';
            elseif (strcmp(char(zephyr_struct.study_metadata(j,6)),'Block') == 1)
                prefix = 'u';
            end
        
            scan_dir = ['./' num2str(j) '-' char(zephyr_struct.study_metadata(j,3))];
            disp(['Normalising: ' num2str(j) '-' char(zephyr_struct.study_metadata(j,3))]);
                    
            % get the mean image and the rest of the EPI images
            mean_image = spm_get('files',scan_dir,['mean' prefix char(zephyr_struct.study_metadata(j,3)) '*.img']);
            image_matrix = spm_get('files',scan_dir,[prefix char(zephyr_struct.study_metadata(j,3)) '*.img']);
                    
            % put the images in the target array
            target_images = char(mean_image,image_matrix);
                    
            spm_write_sn(target_images,parameters,normalise_functional_flags)
                         
        end
    end    

    
disp(' '); disp('Normalisation complete');