function varargout = zephyr_postprocess(varargin)

% ZEPHYR_POSTPROCESS
%
% Version .3 - Beta Software
% Copyright 2004, Craig M. Bennett
%
% --------------------------------------------------------------
%
% Some code snippets from SPM2 used under the GNU GPL
% Copyright 2004 Functional Imaging Laboratory (FIL) 
% The Wellcome Department of Imaging Neuroscience
% Institute of Neurology at University College London
%
% --------------------------------------------------------------
%
% This file is part of Zephyr.
%
% Zephyr is free software; you can redistribute it and/or modify
% it under the terms of the GNU General Public License as published by
% the Free Software Foundation; either version 2 of the License, or
% (at your option) any later version.
%
% Zephyr is distributed in the hope that it will be useful,
% but WITHOUT ANY WARRANTY; without even the implied warranty of
% MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
% GNU General Public License for more details.
%
% You should have received a copy of the GNU General Public License
% along with Zephyr (zephyr_license.txt); if not, write to the Free Software
% Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
%
% --------------------------------------------------------------
%
% zephyr_postprocess.m

% Begin initialization code - DO NOT EDIT
gui_Singleton = 1;
gui_State = struct('gui_Name',       mfilename, ...
                   'gui_Singleton',  gui_Singleton, ...
                   'gui_OpeningFcn', @zephyr_postprocess_OpeningFcn, ...
                   'gui_OutputFcn',  @zephyr_postprocess_OutputFcn, ...
                   'gui_LayoutFcn',  [], ...
                   'gui_Callback',   []);
if nargin & isstr(varargin{1})
    gui_State.gui_Callback = str2func(varargin{1});
end

if nargout
    [varargout{1:nargout}] = gui_mainfcn(gui_State, varargin{:});
else
    gui_mainfcn(gui_State, varargin{:});
end
% End initialization code - DO NOT EDIT


% --- Executes just before zephyr_postprocess is made visible.
function zephyr_postprocess_OpeningFcn(hObject, eventdata, handles, varargin)

    global zephyr_struct;
    global defaults;
	
	spm_defaults;

    subject_axes = findobj('Tag','subject_axes');
    set(subject_axes,'XLim',[0 zephyr_struct.num_subjects]);
    set(subject_axes,'YLim',[0 1]);
    set(subject_axes,'XTickLabel',[]);
    set(subject_axes,'YTickLabel',[]);
    
    progress_axes = findobj('Tag','progress_axes');
    set(progress_axes,'XLim',[0 4]);
    set(progress_axes,'YLim',[0 1]);
    set(progress_axes,'XTickLabel',[]);
    set(progress_axes,'YTickLabel',[]);
    
    drawnow;
    

% Choose default command line output for zephyr_postprocess
handles.output = hObject;

% Update handles structure
guidata(hObject, handles);


% --- Outputs from this function are returned to the command line.
function varargout = zephyr_postprocess_OutputFcn(hObject, eventdata, handles)

% Get default command line output from handles structure
varargout{1} = handles.output;

% --- Executes on button press in postprocess.
function postprocess_Callback(hObject, eventdata, handles)

    global zephyr_struct;
    global defaults;
    
    subject_axes = findobj('Tag','subject_axes');
    progress_axes = findobj('Tag','progress_axes');
    set(findobj(subject_axes,'Tag','subject_bar'),'Xdata',[0 0]);
    
    zephyr_results_folders;
    
    % Complete Modeling for each subject
    for i=1:zephyr_struct.num_subjects

        load([zephyr_struct.data_directory '/zephyr/zephyr_data.mat']);
        
        if (strcmp(char(zephyr_struct.processing_track(i,2)),'Exclude') | strcmp(char(zephyr_struct.processing_track(i,2)),'Done'))
            disp(' '); disp(['Subject ' char(zephyr_struct.processing_track(i,1)) 'excluded from preprocessing.  Skipping...']); disp(' ');
            continue;
           
        elseif (strcmp(char(zephyr_struct.processing_track(i,2)),'Preprocessed'))
            zephyr_struct.processing_track(i,2) = {'Modeling'};
            disp(' '); disp(['Processing ' char(zephyr_struct.processing_track(i,1))]); disp(' ');
            
        end
        
        save([zephyr_struct.data_directory '/zephyr/zephyr_data.mat'],'zephyr_struct')
        
        
	    % cd into the subject folder
	    cd([zephyr_struct.data_directory '/' char(zephyr_struct.processing_track(i,1))]);

        % Update the GUI
        set(findobj('Tag','subject_text'),'String',['Subject: ' char(zephyr_struct.processing_track(i,1))]);
        set(findobj(progress_axes,'Tag','progress_bar'),'Xdata',[0 0]);
        drawnow;
        
        for j=1:zephyr_struct.num_scans
          
            % ---- Prepare the models for each subject
            set(findobj('Tag','progress_text'),'String',['Preparing Models']);
            if (strcmp(char(zephyr_struct.study_metadata(j,1)),'Functional') == 1)
                
                % Copy the study model to the subject's folder
                model_path = [zephyr_struct.data_directory '/results/' num2str(j) '-' char(zephyr_struct.study_metadata(j,3)) '/SPM.mat'];
                subject_model_path = [zephyr_struct.data_directory '/' char(zephyr_struct.processing_track(i,1)) '/results/' num2str(j) '-' char(zephyr_struct.study_metadata(j,3))];
                unix(['cp ' model_path ' ' subject_model_path]);
                
                % cd into the results
	            cd([subject_model_path]);   
                
                % remove mask.img if it exists (resets results)
                if (exist('mask.img'))
                    unix('rm mask.img');    
                end
                
                % Load the SPM.mat file for this scan
                load(['SPM.mat']);
                
                % Get processing parameters from the user's defaults file
                run ../../../zephyr/zephyr_postprocessing_defaults.m
                
                % Find the functional images
                SPM.xY.P = spm_get('files',[zephyr_struct.data_directory '/' char(zephyr_struct.processing_track(i,1)) '/' num2str(j) '-' char(zephyr_struct.study_metadata(j,3))],['swu*.img']);
                
                % keep track of where the subject_model_path is
                SPM.swd = subject_model_path;
                
                % Save the SPM structure
                save(['SPM.mat'],'SPM');
                
                % Prep it for estimation
                spm_fmri_spm_ui(SPM);
                
	            % cd into the subject folder
	            cd([zephyr_struct.data_directory '/' char(zephyr_struct.processing_track(i,1))]);
                
            end
        end
 
        line('Xdata',[0 1], 'Ydata',[0.5 0.5],'LineWidth',6, 'Color', [.8 0 0],'Tag','progress_bar','Parent',progress_axes);
        drawnow;
          
        
        for j=1:zephyr_struct.num_scans

            % ---- Estimate all the models
            set(findobj('Tag','progress_text'),'String',['Estimating']);
            if (strcmp(char(zephyr_struct.study_metadata(j,1)),'Functional') == 1)
                
                subject_results_path = [zephyr_struct.data_directory '/' char(zephyr_struct.processing_track(i,1)) '/results/' num2str(j) '-' char(zephyr_struct.study_metadata(j,3))];

                % cd into the results
	            cd([subject_results_path]); 
                
                % Load the SPM.mat file for this scan
                load(['SPM.mat']);
                
                spm_spm(SPM);
    
            end
        end
        
        line('Xdata',[0 2], 'Ydata',[0.5 0.5],'LineWidth',6, 'Color', [.8 0 0],'Tag','progress_bar','Parent',progress_axes);
		drawnow;
       
        
        for j=1:zephyr_struct.num_scans

            % ---- Compute the contrasts
            set(findobj('Tag','progress_text'),'String',['Evaluating Contrasts']);
            if (strcmp(char(zephyr_struct.study_metadata(j,1)),'Functional') == 1)
                
                subject_results_path = [zephyr_struct.data_directory '/' char(zephyr_struct.processing_track(i,1)) '/results/' num2str(j) '-' char(zephyr_struct.study_metadata(j,3))];
                
                % cd into the results
	            cd([subject_results_path]); 
                
                % Load the SPM.mat file for this scan
                load(['SPM.mat']); 
                
                spm_contrasts(SPM);
                
            end
        end
        
        line('Xdata',[0 3], 'Ydata',[0.5 0.5],'LineWidth',6, 'Color', [.8 0 0],'Tag','progress_bar','Parent',progress_axes);
		drawnow;
        
        
        for j=1:zephyr_struct.num_scans

            % ---- move the contrasts
            set(findobj('Tag','progress_text'),'String',['Evaluating Contrasts']);
            if (strcmp(char(zephyr_struct.study_metadata(j,1)),'Functional') == 1)
                
                subject_results_path = [zephyr_struct.data_directory '/' char(zephyr_struct.processing_track(i,1)) '/results/' num2str(j) '-' char(zephyr_struct.study_metadata(j,3))];
                main_results_path = [zephyr_struct.data_directory '/results/' num2str(j) '-' char(zephyr_struct.study_metadata(j,3))];
                
                % cd into the results
	            cd([subject_results_path]); 
                
                % Load the SPM.mat file for this scan
                load(['SPM.mat']); 
                
                for t = 1:length(SPM.Sess.U)
                    
                    if (t < 10)
                        prefix = '000';
                    elseif (t < 100)
                        prefix = '00';
                    elseif (t < 1000)
                        prefix = '0';
                    elseif (t < 10000)
                        disp('Wow, you have a lot of conditions');
                        prefix = '';
                    end
                    
                    unix(['cp spmF_' prefix num2str(t) '.hdr ' main_results_path '/' char(SPM.Sess.U(t).name) '/' char(zephyr_struct.processing_track(i,1)) '.hdr']);
                    unix(['cp spmF_' prefix num2str(t) '.img ' main_results_path '/' char(SPM.Sess.U(t).name) '/' char(zephyr_struct.processing_track(i,1)) '.img']);
                    
                end    
                
            end
        end
        
        
        line('Xdata',[0 4], 'Ydata',[0.5 0.5],'LineWidth',6, 'Color', [.8 0 0],'Tag','progress_bar','Parent',progress_axes);
		drawnow;
        
        line('Xdata',[0 i], 'Ydata',[0.5 0.5],'LineWidth',6, 'Color', [.8 0 0],'Tag','subject_bar','Parent',subject_axes);
		drawnow;
        
    end
    
    set(findobj('Tag','subject_text'),'String',['Complete']);
    set(findobj('Tag','progress_text'),'String',['']);



function zephyr_results_folders()

    global zephyr_struct
    
    warning off MATLAB:MKDIR:DirectoryExists
    
    for j=1:zephyr_struct.num_scans
        if (strcmp(char(zephyr_struct.study_metadata(j,1)),'Functional') == 1)
            
            results_path = [zephyr_struct.data_directory '/results/' num2str(j) '-' char(zephyr_struct.study_metadata(j,3))];
            cd(results_path)
            load(['SPM']);
            
            for t = 1:length(SPM.Sess.U)
                    mkdir([char(SPM.Sess.U(t).name)]);
            end    

        end
    end
    




