function varargout = zephyr_preprocess(varargin)

% ZEPHYR_PREPROCESS
%
% Version .3 - Beta Software
% Copyright 2004, Craig M. Bennett
%
% --------------------------------------------------------------
%
% Some code snippets from SPM2 used under the GNU GPL
% Copyright 2004 Functional Imaging Laboratory (FIL) 
% The Wellcome Department of Imaging Neuroscience
% Institute of Neurology at University College London
%
% --------------------------------------------------------------
%
% This file is part of Zephyr.
%
% Zephyr is free software; you can redistribute it and/or modify
% it under the terms of the GNU General Public License as published by
% the Free Software Foundation; either version 2 of the License, or
% (at your option) any later version.
%
% Zephyr is distributed in the hope that it will be useful,
% but WITHOUT ANY WARRANTY; without even the implied warranty of
% MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
% GNU General Public License for more details.
%
% You should have received a copy of the GNU General Public License
% along with Zephyr (zephyr_license.txt); if not, write to the Free Software
% Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
%
% --------------------------------------------------------------
%
% zephyr_preprocess.m

% Begin initialization code - DO NOT EDIT
gui_Singleton = 1;
gui_State = struct('gui_Name',       mfilename, ...
                   'gui_Singleton',  gui_Singleton, ...
                   'gui_OpeningFcn', @zephyr_preprocess_OpeningFcn, ...
                   'gui_OutputFcn',  @zephyr_preprocess_OutputFcn, ...
                   'gui_LayoutFcn',  [], ...
                   'gui_Callback',   []);
if nargin & isstr(varargin{1})
    gui_State.gui_Callback = str2func(varargin{1});
end

if nargout
    [varargout{1:nargout}] = gui_mainfcn(gui_State, varargin{:});
else
    gui_mainfcn(gui_State, varargin{:});
end
% End initialization code - DO NOT EDIT


% --- Executes just before zephyr_preprocess is made visible.
function zephyr_preprocess_OpeningFcn(hObject, eventdata, handles, varargin)

    global zephyr_struct;

    subject_axes = findobj('Tag','subject_axes');
    set(subject_axes,'XLim',[0 zephyr_struct.num_subjects]);
    set(subject_axes,'YLim',[0 1]);
    set(subject_axes,'XTickLabel',[]);
    set(subject_axes,'YTickLabel',[]);
    
    progress_axes = findobj('Tag','progress_axes');
    set(progress_axes,'XLim',[0 9]);
    set(progress_axes,'YLim',[0 1]);
    set(progress_axes,'XTickLabel',[]);
    set(progress_axes,'YTickLabel',[]);
    
    drawnow;
    

% Choose default command line output for zephyr_preprocess
handles.output = hObject;

% Update handles structure
guidata(hObject, handles);


% --- Outputs from this function are returned to the command line.
function varargout = zephyr_preprocess_OutputFcn(hObject, eventdata, handles)

% Get default command line output from handles structure
varargout{1} = handles.output;


% --- Executes on button press in preprocess.
function preprocess_Callback(hObject, eventdata, handles)

    global zephyr_struct;
    subject_axes = findobj('Tag','subject_axes');
    progress_axes = findobj('Tag','progress_axes');
    set(findobj(subject_axes,'Tag','subject_bar'),'Xdata',[0 0]);
    
    
    % reset stuck subjects
    load([zephyr_struct.data_directory '/zephyr/zephyr_data.mat']);
    
        for p=1:zephyr_struct.num_subjects
            if (strcmp(char(zephyr_struct.processing_track(p,2)),'Processing'))
                zephyr_struct.processing_track(p,2) = {'-'};
            end    
        end
    
    save([zephyr_struct.data_directory '/zephyr/zephyr_data.mat'],'zephyr_struct')
    
    
    % Create the root results & models folders
    warning off MATLAB:MKDIR:DirectoryExists
    cd([zephyr_struct.data_directory]);
    
    try
        mkdir('./results');
        mkdir('./models');
    end
    
    
    % Create the root results & models subfolders
    for j=1:zephyr_struct.num_scans
        if (strcmp(char(zephyr_struct.study_metadata(j,1)),'Functional') == 1)
            try
                mkdir(['./results/' num2str(j) '-' char(zephyr_struct.study_metadata(j,3))]);
                mkdir(['./models/' num2str(j) '-' char(zephyr_struct.study_metadata(j,3))]);
            end
        end
    end
    
    
    % the main preprocessing loop - do the following steps for all subjects
    for i=1:zephyr_struct.num_subjects
        
        load([zephyr_struct.data_directory '/zephyr/zephyr_data.mat']);
        
        if (strcmp(char(zephyr_struct.processing_track(i,2)),'Exclude') | strcmp(char(zephyr_struct.processing_track(i,2)),'Preprocessed') | strcmp(char(zephyr_struct.processing_track(i,2)),'Done'))
            disp(' '); disp(['Subject ' char(zephyr_struct.processing_track(i,1)) 'excluded from preprocessing.  Skipping...']); disp(' ');
            continue;
           
        else
            zephyr_struct.processing_track(i,2) = {'Processing'};
            disp(' '); disp(['Processing ' char(zephyr_struct.processing_track(i,1))]); disp(' ');
        end
        
        save([zephyr_struct.data_directory '/zephyr/zephyr_data.mat'],'zephyr_struct')

        
        set(findobj('Tag','subject_text'),'String',['Subject: ' char(zephyr_struct.processing_track(i,1))]);
        set(findobj(progress_axes,'Tag','progress_bar'),'Xdata',[0 0]);
        drawnow;
        
	    % cd into the subject folder
	    cd([zephyr_struct.data_directory '/' char(zephyr_struct.processing_track(i,1))]);
        
        % create the necessary folders
        set(findobj('Tag','progress_text'),'String',['Organizing Directory']);
        drawnow;
        zephyr_create_folders
        line('Xdata',[0 1], 'Ydata',[0.5 0.5],'LineWidth',6, 'Color', [.8 0 0],'Tag','progress_bar','Parent',progress_axes);
		drawnow;

        % convert the raw data
        set(findobj('Tag','progress_text'),'String',['Converting Raw Data']);
        drawnow;
        zephyr_GEConvert
        line('Xdata',[0 2], 'Ydata',[0.5 0.5],'LineWidth',6, 'Color', [.8 0 0],'Tag','progress_bar','Parent',progress_axes);
		drawnow;        
        
        % correct slice timing for event-related functional scans
        set(findobj('Tag','progress_text'),'String',['Slice Timing']);
        drawnow;
        zephyr_slice_timing;
        line('Xdata',[0 3], 'Ydata',[0.5 0.5],'LineWidth',6, 'Color', [.8 0 0],'Tag','progress_bar','Parent',progress_axes);
		drawnow;  
        
        % realign & unwarp functional scans
        set(findobj('Tag','progress_text'),'String',['Realigning']);
        drawnow;
        zephyr_realign_unwarp;
        line('Xdata',[0 4], 'Ydata',[0.5 0.5],'LineWidth',6, 'Color', [.8 0 0],'Tag','progress_bar','Parent',progress_axes);
		drawnow;          
        
        % coregister every scan in existence...
        set(findobj('Tag','progress_text'),'String',['Coregistering']);
        drawnow;
        zephyr_coregister;
        line('Xdata',[0 6], 'Ydata',[0.5 0.5],'LineWidth',6, 'Color', [.8 0 0],'Tag','progress_bar','Parent',progress_axes);
		drawnow;    
        
        % then normalise them...
        set(findobj('Tag','progress_text'),'String',['Normalizing']);
        drawnow;
        zephyr_normalise;
        line('Xdata',[0 8], 'Ydata',[0.5 0.5],'LineWidth',6, 'Color', [.8 0 0],'Tag','progress_bar','Parent',progress_axes);
		drawnow;    
        
        % smoooooth...
        set(findobj('Tag','progress_text'),'String',['Smoothing']);
        drawnow;
        zephyr_smooth;
        line('Xdata',[0 9], 'Ydata',[0.5 0.5],'LineWidth',6, 'Color', [.8 0 0],'Tag','progress_bar','Parent',progress_axes);
		drawnow;    
        
        % indicate this subject finished
        line('Xdata',[0 i], 'Ydata',[0.5 0.5],'LineWidth',6, 'Color', [.8 0 0],'Tag','subject_bar','Parent',subject_axes);
        drawnow;
        
        % Update zephyr.processing_track
        load([zephyr_struct.data_directory '/zephyr/zephyr_data.mat']);
            zephyr_struct.processing_track(i,2) = {'Preprocessed'};
        save([zephyr_struct.data_directory '/zephyr/zephyr_data.mat'],'zephyr_struct')
        
        
    end
    
    set(findobj('Tag','subject_text'),'String',['Complete']);
    set(findobj('Tag','progress_text'),'String',['']);

    
    
    
    
    
function zephyr_create_folders()

    global zephyr_struct;
    
    warning off MATLAB:MKDIR:DirectoryExists
    
    % rename the data folder 'raw'
        if (exist('raw','dir') ~= 7)
		    unix('mv * raw');
	    end
    
    % Create the results & models folder
    try
        mkdir('./results');
    end
    
    for j=1:zephyr_struct.num_scans
        
            if (strcmp(char(zephyr_struct.study_metadata(j,1)),'Coplanar') == 1)
                try
                    mkdir(['./' num2str(j) '-Coplanar']);
                end
                
            elseif (strcmp(char(zephyr_struct.study_metadata(j,1)),'Functional') == 1)
                try
                    mkdir(['./' num2str(j) '-' char(zephyr_struct.study_metadata(j,3))]);
                    mkdir(['./results/' num2str(j) '-' char(zephyr_struct.study_metadata(j,3))]);
                end
                
            elseif (strcmp(char(zephyr_struct.study_metadata(j,1)),'Hires') == 1)
                try
                    mkdir(['./' num2str(j) '-Hires']);
                end
                
            elseif (strcmp(char(zephyr_struct.study_metadata(j,1)),'DTI') == 1)   
                try
                    mkdir(['./' num2str(j) '-DTI']);
                end
                
            end
            
    end

    
    
    
    
function zephyr_GEConvert()

    global zephyr_struct;
    global defaults;
                
    defaults.analyze.flip = 1;
                
    for j=1:zephyr_struct.num_scans
        
            if (strcmp(char(zephyr_struct.study_metadata(j,1)),'Coplanar') == 1)
                if (j < 10)
				    prefix = '00';
			    else
				    prefix = '0';
			    end
            
                eval(['GE_convertVolume(''./raw/' prefix,num2str(j) ''',1,''./' num2str(j) '-Coplanar/coplanar'')']);
                
            elseif (strcmp(char(zephyr_struct.study_metadata(j,1)),'Functional') == 1)
                if (j < 10) 
                    prefix = '00';
                else
                    prefix = '0';
                end
                
                TR = str2num(char(zephyr_struct.study_metadata(j,5)));
                
                global defaults;
                
                defaults.analyze.flip = 1;
                
                eval(['GE_convertADW(''./raw/' prefix,num2str(j) ''',''./' num2str(j) '-' char(zephyr_struct.study_metadata(j,3)) '/' char(zephyr_struct.study_metadata(j,3)) ''',2,' num2str(TR) ')']);
                
                
            elseif (strcmp(char(zephyr_struct.study_metadata(j,1)),'Hires') == 1)
                if (j < 10)
				    prefix = '00';
			    else
				    prefix = '0';
			    end
            
                eval(['GE_convertVolume(''./raw/' prefix,num2str(j) ''',1,''./' num2str(j) '-Hires/hires'')']);
                
            elseif (strcmp(char(zephyr_struct.study_metadata(j,1)),'DTI') == 1)   
                if (j < 10) 
                    prefix = '00';
                else
                    prefix = '0';
                end
                
                eval(['GE_convertADW(''./raw/' prefix,num2str(j) ''',''./' num2str(j) '-DTI/dti'',1,7)']);
                
            end
            
    end
