function varargout = zephyr_roi(varargin)

% ZEPHYR_ROI M-file for zephyr_roi.fig
%
% Version .3 - Beta Software
% Copyright 2004, Craig M. Bennett
%
% --------------------------------------------------------------
%
% Some code snippets from SPM2 used under the GNU GPL
% Copyright 2004 Functional Imaging Laboratory (FIL) 
% The Wellcome Department of Imaging Neuroscience
% Institute of Neurology at University College London
%
% --------------------------------------------------------------
%
% This file is part of Zephyr.
%
% Zephyr is free software; you can redistribute it and/or modify
% it under the terms of the GNU General Public License as published by
% the Free Software Foundation; either version 2 of the License, or
% (at your option) any later version.
%
% Zephyr is distributed in the hope that it will be useful,
% but WITHOUT ANY WARRANTY; without even the implied warranty of
% MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
% GNU General Public License for more details.
%
% You should have received a copy of the GNU General Public License
% along with Zephyr (zephyr_license.txt); if not, write to the Free Software
% Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
%
% --------------------------------------------------------------
%
% zephyr_roi.m

% Begin initialization code - DO NOT EDIT
gui_Singleton = 1;
gui_State = struct('gui_Name',       mfilename, ...
                   'gui_Singleton',  gui_Singleton, ...
                   'gui_OpeningFcn', @zephyr_roi_OpeningFcn, ...
                   'gui_OutputFcn',  @zephyr_roi_OutputFcn, ...
                   'gui_LayoutFcn',  [] , ...
                   'gui_Callback',   []);
if nargin & isstr(varargin{1})
    gui_State.gui_Callback = str2func(varargin{1});
end

if nargout
    [varargout{1:nargout}] = gui_mainfcn(gui_State, varargin{:});
else
    gui_mainfcn(gui_State, varargin{:});
end
% End initialization code - DO NOT EDIT


% --- Executes just before zephyr_roi is made visible.
function zephyr_roi_OpeningFcn(hObject, eventdata, handles, varargin)
% This function has no output args, see OutputFcn.
% hObject    handle to figure
% eventdata  reserved - to be defined in a future version of MATLAB
% handles    structure with handles and user data (see GUIDATA)
% varargin   command line arguments to zephyr_roi (see VARARGIN)

handles.condition = '';

% Choose default command line output for zephyr_roi
handles.output = hObject;

% Update handles structure
guidata(hObject, handles);

% UIWAIT makes zephyr_roi wait for user response (see UIRESUME)
% uiwait(handles.figure1);


% --- Outputs from this function are returned to the command line.
function varargout = zephyr_roi_OutputFcn(hObject, eventdata, handles)
% varargout  cell array for returning output args (see VARARGOUT);
% hObject    handle to figure
% eventdata  reserved - to be defined in a future version of MATLAB
% handles    structure with handles and user data (see GUIDATA)

% Get default command line output from handles structure
varargout{1} = handles.output;



% --- Executes on button press in process_roi.
function process_roi_Callback(hObject, eventdata, handles)
    
    disp(' ');
    
    % copy the necessary variables from main workspace
    SPM = evalin('base','SPM');
    xSPM = evalin('base','xSPM');
    hReg = evalin('base','hReg');
    
    % go to the current results directory
    cd(xSPM.swd);
    
    % if there is no ROI folder then make one
    roi_dir = ['ROI-' handles.condition];
    if (exist(roi_dir,'dir') ~= 7)
        mkdir(roi_dir);    
    end

    % move the existing spm2.ps file, if it exists
    if (exist('spm2.ps','file') ~= 0)
            unix('mv spm2.ps spm2-1.ps');
    end    
    
    % let SPM find all the clusters & voxels
    cluster_data = spm_list('List',xSPM,hReg);
    spm_print;
    unix(['mv spm2.ps VOI_overview.ps']);
    
    % get the count of cluster-level groups
    num_cluster_level = cluster_data.dat(1,2);
    num_cluster_level = num_cluster_level{1};
    
    % get the count of voxel-level groups
    num_voxel_level = size(cluster_data.dat);
    num_voxel_level = num_voxel_level(1,1);
        
    disp(['Number of cluster-level groups: ' int2str(num_cluster_level)]);
    disp(['Number of voxel-level groups: ' int2str(num_voxel_level)]);
    disp(' ');

    
    for i = 1:num_voxel_level
        
        % get cluster info
        cluster_check = cluster_data.dat(i,4);
        cluster_check = cluster_check{1};
        
        % proceed if this is a cluster group
        if (cluster_check > 0)
    
            if (i < 10)
                name = ['0' int2str(i)];
            else
                name = [int2str(i)];
            end
            
            new_location = cluster_data.dat(i,11);
            new_location = new_location{1};
            
            disp(['Working: Cluster ' name]);
           
            clear xY;
            xY.xyz = new_location;
            xY.name = name;
            xY.Ic = 0;
            xY.def = 'cluster';
            
            [Y,xY] = spm_regions(xSPM,SPM,hReg,xY);
            
            spm_print;
            unix(['mv spm2.ps VOI_' name '.ps']);
            
            %---write raw cluster data
            filename = ['VOI_' name '_region.txt'];
            FID = fopen(filename,'w');
            
                fprintf(FID,'Region of Interest %s\n\n',name);
                fprintf(FID,'Image\t\tValue\n');
            
                num_values = size(xY.s);
                num_values = num_values(1);
                
                for j = 1:num_values
                    fprintf(FID,'%i\t\t%f\n',j,Y(j));    
                end
            
            fclose(FID);
            
            %---write raw voxel data
            filename = ['VOI_' name '_voxels.txt'];
            FID = fopen(filename,'w');
            
                fprintf(FID,'Voxels of Interest %s\n\n',name);
                fprintf(FID,'Voxel\tImage Number\n');
            
                num_voxels = size(xY.v);
                num_voxels = num_voxels(1);
                
                fprintf(FID,'XYZmm\t');
                for l = 1:num_values
                    fprintf(FID,'%i\t',l);    
                end
                fprintf(FID,'\n');
               
                for k = 1:num_voxels   
                    
                    fprintf(FID,'%s\t',mat2str(xY.XYZmm(:,k)));
                    for l = 1:num_values
                        fprintf(FID,'%f\t',xY.y(l,k));    
                    end
                    fprintf(FID,'\n',k);
                    
                end
            
            fclose(FID);           
            
            %---end file writing

        end
    end

    %move files to ROI directory
    unix(['mv VOI* ./' roi_dir '/']);
    
    % move the existing spm2.ps file back, if it exists
    if (exist('spm2-1.ps','file') == 2)
            unix('mv spm2-1.ps spm2.ps');
    end  
   
    disp(' ');
    disp('Completed');
    disp(' ');
    
    close('Zephyr ROI');



% --- Executes during object creation, after setting all properties.
function condition_name_CreateFcn(hObject, eventdata, handles)

    set(hObject,'BackgroundColor','white');


function condition_name_Callback(hObject, eventdata, handles)

    handles.condition = get(hObject,'String');

    % Update handles structure
    guidata(hObject, handles);