function varargout = zephyr_scan_parameters(varargin)

% ZEPHYR_SCAN_PARAMETERS M-file for zephyr_scan_parameters.fig
%
% Version .3 - Beta Software
% Copyright 2004, Craig M. Bennett
%
% --------------------------------------------------------------
%
% Some code snippets from SPM2 used under the GNU GPL
% Copyright 2004 Functional Imaging Laboratory (FIL) 
% The Wellcome Department of Imaging Neuroscience
% Institute of Neurology at University College London
%
% --------------------------------------------------------------
%
% This file is part of Zephyr.
%
% Zephyr is free software; you can redistribute it and/or modify
% it under the terms of the GNU General Public License as published by
% the Free Software Foundation; either version 2 of the License, or
% (at your option) any later version.
%
% Zephyr is distributed in the hope that it will be useful,
% but WITHOUT ANY WARRANTY; without even the implied warranty of
% MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
% GNU General Public License for more details.
%
% You should have received a copy of the GNU General Public License
% along with Zephyr (zephyr_license.txt); if not, write to the Free Software
% Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
%
% --------------------------------------------------------------
%
% zephyr_scan_parameters.m

% Begin initialization code - DO NOT EDIT
gui_Singleton = 1;
gui_State = struct('gui_Name',       mfilename, ...
                   'gui_Singleton',  gui_Singleton, ...
                   'gui_OpeningFcn', @zephyr_scan_parameters_OpeningFcn, ...
                   'gui_OutputFcn',  @zephyr_scan_parameters_OutputFcn, ...
                   'gui_LayoutFcn',  [] , ...
                   'gui_Callback',   []);
if nargin & isstr(varargin{1})
    gui_State.gui_Callback = str2func(varargin{1});
end

if nargout
    [varargout{1:nargout}] = gui_mainfcn(gui_State, varargin{:});
else
    gui_mainfcn(gui_State, varargin{:});
end
% End initialization code - DO NOT EDIT


% --- Executes just before zephyr_scan_parameters is made visible.
function zephyr_scan_parameters_OpeningFcn(hObject, eventdata, handles, varargin)

global zephyr_struct;

handles.iteration = cell2mat(varargin(1));

set(findobj('Tag','scan_num_text'),'String',['Scan ' num2str(handles.iteration)]);


% Choose default command line output for zephyr_scan_parameters
handles.output = hObject;

% Update handles structure
guidata(hObject, handles);


% --- Outputs from this function are returned to the command line.
function varargout = zephyr_scan_parameters_OutputFcn(hObject, eventdata, handles)

% Get default command line output from handles structure
varargout{1} = handles.output;


% --- Executes on button press in enter.
function enter_Callback(hObject, eventdata, handles)
    
    global zephyr_struct;
    
    cd(zephyr_struct.data_directory);
    
    switch char(zephyr_struct.study_metadata(handles.iteration,1))
        case 'Scout'
            zephyr_struct.scan_types.scout = 1;
            
        case 'Coplanar'
            zephyr_struct.scan_types.coplanar = 1;
            
        case 'Functional'
            zephyr_struct.scan_types.functional = 1;
            
        case 'Hires'
            zephyr_struct.scan_types.hires = 1;
            
        case 'DTI'
            zephyr_struct.scan_types.dti = 1;
            
    end
    
    save('./zephyr/zephyr_data.mat','zephyr_struct');
    
    disp(' '); disp(['Data written for scan ' num2str(handles.iteration)  '...']); disp(' ');

    close;

   


% --- Executes during object creation, after setting all properties.
function scan_type_CreateFcn(hObject, eventdata, handles)

set(hObject,'BackgroundColor',get(0,'defaultUicontrolBackgroundColor'));

% --- Executes on selection change in scan_type.
function scan_type_Callback(hObject, eventdata, handles)

    global zephyr_struct;
    
    val = get(hObject,'Value');
    str = get(hObject,'String');
    
    zephyr_struct.study_metadata(handles.iteration,1:6) = {''};
    zephyr_struct.study_metadata(handles.iteration,1) = {str{val}};
    
    switch str{val}
        case 'Scout'
            set(findobj('Tag','func_name_text'),'Visible','off');
            set(findobj('Tag','func_name_enter'),'Visible','off');
            
            set(findobj('Tag','slice_num_text'),'Visible','off');
            set(findobj('Tag','slice_num_enter'),'Visible','off');
            
            set(findobj('Tag','tr_length_text'),'Visible','off');
            set(findobj('Tag','tr_length_enter'),'Visible','off');
            
            set(findobj('Tag','num_trs_text'),'Visible','off');
            set(findobj('Tag','num_trs_enter'),'Visible','off');
            
            set(findobj('Tag','event_block_text'),'Visible','off');
            set(findobj('Tag','event_block_enter'),'Visible','off');
            
        case 'Coplanar'
            set(findobj('Tag','func_name_text'),'Visible','off');
            set(findobj('Tag','func_name_enter'),'Visible','off');
            
            set(findobj('Tag','slice_num_text'),'Visible','on');
            set(findobj('Tag','slice_num_enter'),'Visible','on');
            
            set(findobj('Tag','tr_length_text'),'Visible','off');
            set(findobj('Tag','tr_length_enter'),'Visible','off');
            
            set(findobj('Tag','num_trs_text'),'Visible','off');
            set(findobj('Tag','num_trs_enter'),'Visible','off');
            
            set(findobj('Tag','event_block_text'),'Visible','off');
            set(findobj('Tag','event_block_enter'),'Visible','off');
            
            
        case 'Functional'
            set(findobj('Tag','func_name_text'),'Visible','on');
            set(findobj('Tag','func_name_enter'),'Visible','on');
            
            set(findobj('Tag','slice_num_text'),'Visible','on');
            set(findobj('Tag','slice_num_enter'),'Visible','on');
            
            set(findobj('Tag','tr_length_text'),'Visible','on');
            set(findobj('Tag','tr_length_enter'),'Visible','on');
            
            set(findobj('Tag','num_trs_text'),'Visible','on');
            set(findobj('Tag','num_trs_enter'),'Visible','on');
            
            set(findobj('Tag','event_block_text'),'Visible','on');
            set(findobj('Tag','event_block_enter'),'Visible','on');            
            
        case 'Hires'
            set(findobj('Tag','func_name_text'),'Visible','off');
            set(findobj('Tag','func_name_enter'),'Visible','off');
            
            set(findobj('Tag','slice_num_text'),'Visible','off');
            set(findobj('Tag','slice_num_enter'),'Visible','off');
            
            set(findobj('Tag','tr_length_text'),'Visible','off');
            set(findobj('Tag','tr_length_enter'),'Visible','off');
            
            set(findobj('Tag','num_trs_text'),'Visible','off');
            set(findobj('Tag','num_trs_enter'),'Visible','off');
            
            set(findobj('Tag','event_block_text'),'Visible','off');
            set(findobj('Tag','event_block_enter'),'Visible','off');            
            
            
        case 'DTI'
            set(findobj('Tag','func_name_text'),'Visible','off');
            set(findobj('Tag','func_name_enter'),'Visible','off');
            
            set(findobj('Tag','slice_num_text'),'Visible','off');
            set(findobj('Tag','slice_num_enter'),'Visible','off');
            
            set(findobj('Tag','tr_length_text'),'Visible','off');
            set(findobj('Tag','tr_length_enter'),'Visible','off');
            
            set(findobj('Tag','num_trs_text'),'Visible','off');
            set(findobj('Tag','num_trs_enter'),'Visible','off');
            
            set(findobj('Tag','event_block_text'),'Visible','off');
            set(findobj('Tag','event_block_enter'),'Visible','off');

    end

    

% --- Executes during object creation, after setting all properties.
function func_name_enter_CreateFcn(hObject, eventdata, handles)

    set(hObject,'BackgroundColor','white');

function func_name_enter_Callback(hObject, eventdata, handles)

    global zephyr_struct;
    
    zephyr_struct.study_metadata(handles.iteration,3) = {get(hObject,'String')};


% --- Executes during object creation, after setting all properties.
function slice_num_enter_CreateFcn(hObject, eventdata, handles)

set(hObject,'BackgroundColor','white');

function slice_num_enter_Callback(hObject, eventdata, handles)

    global zephyr_struct;
    
    zephyr_struct.study_metadata(handles.iteration,2) = {get(hObject,'String')};

    
% --- Executes during object creation, after setting all properties.
function tr_length_enter_CreateFcn(hObject, eventdata, handles)

    set(hObject,'BackgroundColor','white');

function tr_length_enter_Callback(hObject, eventdata, handles)

    global zephyr_struct;
    
    zephyr_struct.study_metadata(handles.iteration,4) = {get(hObject,'String')};

    
% --- Executes during object creation, after setting all properties.
function num_trs_enter_CreateFcn(hObject, eventdata, handles)

    set(hObject,'BackgroundColor','white');

function num_trs_enter_Callback(hObject, eventdata, handles)

    global zephyr_struct;
    
    zephyr_struct.study_metadata(handles.iteration,5) = {get(hObject,'String')};

    
% --- Executes during object creation, after setting all properties.
function event_block_enter_CreateFcn(hObject, eventdata, handles)

    set(hObject,'BackgroundColor',get(0,'defaultUicontrolBackgroundColor'));

% --- Executes on selection change in event_block_enter.
function event_block_enter_Callback(hObject, eventdata, handles)

    global zephyr_struct;
    
    val = get(hObject,'Value');
    str = get(hObject,'String');
    
    zephyr_struct.study_metadata(handles.iteration,6) = {str{val}};


