function varargout = zephyr_test(varargin)

% ZEPHYR_TEST
%
% Version .3 - Beta Software
% Copyright 2004, Craig M. Bennett
%
% --------------------------------------------------------------
%
% Some code snippets from SPM2 used under the GNU GPL
% Copyright 2004 Functional Imaging Laboratory (FIL) 
% The Wellcome Department of Imaging Neuroscience
% Institute of Neurology at University College London
%
% --------------------------------------------------------------
%
% This file is part of Zephyr.
%
% Zephyr is free software; you can redistribute it and/or modify
% it under the terms of the GNU General Public License as published by
% the Free Software Foundation; either version 2 of the License, or
% (at your option) any later version.
%
% Zephyr is distributed in the hope that it will be useful,
% but WITHOUT ANY WARRANTY; without even the implied warranty of
% MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
% GNU General Public License for more details.
%
% You should have received a copy of the GNU General Public License
% along with Zephyr (zephyr_license.txt); if not, write to the Free Software
% Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
%
% --------------------------------------------------------------
%
% zephyr_test.m

% Begin initialization code - DO NOT EDIT
gui_Singleton = 1;
gui_State = struct('gui_Name',       mfilename, ...
                   'gui_Singleton',  gui_Singleton, ...
                   'gui_OpeningFcn', @zephyr_test_OpeningFcn, ...
                   'gui_OutputFcn',  @zephyr_test_OutputFcn, ...
                   'gui_LayoutFcn',  [], ...
                   'gui_Callback',   []);
if nargin & isstr(varargin{1})
    gui_State.gui_Callback = str2func(varargin{1});
end

if nargout
    [varargout{1:nargout}] = gui_mainfcn(gui_State, varargin{:});
else
    gui_mainfcn(gui_State, varargin{:});
end
% End initialization code - DO NOT EDIT


% --- Executes just before zephyr_test is made visible.
function zephyr_test_OpeningFcn(hObject, eventdata, handles, varargin)

    global zephyr_struct;

    subject_axes = findobj('Tag','subject_axes');
    set(subject_axes,'XLim',[0 zephyr_struct.num_subjects]);
    set(subject_axes,'YLim',[0 1]);
    set(subject_axes,'XTickLabel',[]);
    set(subject_axes,'YTickLabel',[]);
    
    progress_axes = findobj('Tag','progress_axes');
    set(progress_axes,'XLim',[0 zephyr_struct.num_scans]);
    set(progress_axes,'YLim',[0 1]);
    set(progress_axes,'XTickLabel',[]);
    set(progress_axes,'YTickLabel',[]);
    
    drawnow;
    

% Choose default command line output for zephyr_test
handles.output = hObject;

% Update handles structure
guidata(hObject, handles);


% --- Outputs from this function are returned to the command line.
function varargout = zephyr_test_OutputFcn(hObject, eventdata, handles)

% Get default command line output from handles structure
varargout{1} = handles.output;

% --- Executes on button press in test.
function test_Callback(hObject, eventdata, handles)

    global zephyr_struct;
    subject_axes = findobj('Tag','subject_axes');
    progress_axes = findobj('Tag','progress_axes');
    set(findobj(subject_axes,'Tag','subject_bar'),'Xdata',[0 0]);
    
    for i=1:zephyr_struct.num_subjects

	    % cd into the subject folder
	    cd([zephyr_struct.data_directory '/' char(zephyr_struct.processing_track(i,1))]);
        
        % rename the data folder 'raw'
        if (exist('raw','dir') ~= 7)
		    unix('mv * raw');
	    end

        set(findobj('Tag','subject_text'),'String',['Subject: ' char(zephyr_struct.processing_track(i,1))]);
        set(findobj(progress_axes,'Tag','progress_bar'),'Xdata',[0 0]);
        drawnow;
        
        for j=1:zephyr_struct.num_scans
            set(findobj('Tag','progress_text'),'String',['Scan: ' num2str(j)]);
            
            if (strcmp(char(zephyr_struct.study_metadata(j,1)),'Scout') == 1)
                zephyr_scout_test(i,j)
            elseif (strcmp(char(zephyr_struct.study_metadata(j,1)),'Coplanar') == 1)
                zephyr_coplanar_test(i,j)
            elseif (strcmp(char(zephyr_struct.study_metadata(j,1)),'Functional') == 1)
                zephyr_functional_test(i,j)
            elseif (strcmp(char(zephyr_struct.study_metadata(j,1)),'Hires') == 1)
                zephyr_hires_test(i,j)
            elseif (strcmp(char(zephyr_struct.study_metadata(j,1)),'DTI') == 1)   
                zephyr_dti_test(i,j)
            end
            
            line('Xdata',[0 j], 'Ydata',[0.5 0.5],'LineWidth',6, 'Color', [.8 0 0],'Tag','progress_bar','Parent',progress_axes);
		    drawnow;
            
        end
        
        line('Xdata',[0 i], 'Ydata',[0.5 0.5],'LineWidth',6, 'Color', [.8 0 0],'Tag','subject_bar','Parent',subject_axes);
		drawnow;
        
    end
    
    set(findobj('Tag','subject_text'),'String',['Complete']);
    set(findobj('Tag','progress_text'),'String',['']);













function zephyr_scout_test(i,j)

    global zephyr_struct
    
    subject_error = 0;
    path_error = 0;
    file_error = 0;
    
    if (j < 10)
        prefix = '00';
    else
        prefix = '0';
    end		
    
    test_path = ['./raw/' prefix,int2str(j)];

    if (exist(test_path,'dir') ~= 7)
		path_error = 1;
		subject_error = 1;	
        
    else
        [a,b] = unix(['ls ', test_path, ' | wc -l | awk ''{print $1}''']);
        file_count = str2num(b);

        % scout is 9 slices
        slices = 9;
        
        if (file_count < (slices + 1))
		    file_error = 1;
			subject_error = 1;
        elseif (file_count > (slices + 1))
            file_error = 2;
			subject_error = 1;
		end
        
	end
    
    if (subject_error == 1)
        
        if (path_error == 1)
            choice = questdlg(['(' char(zephyr_struct.processing_track(i,1)) ') Scan ' num2str(j) ':Scout - Path does not exist'],'Scout Error','Process Subject','Exclude Subject','Exclude Subject');    
        elseif (file_error == 1)
            choice = questdlg(['[' char(zephyr_struct.processing_track(i,1)) '] Scan ' num2str(j) ':Scout - Files are missing'],'Scout Error','Process Subject','Exclude Subject','Exclude Subject');
        elseif (file_error == 2)
            choice = questdlg(['[' char(zephyr_struct.processing_track(i,1)) '] Scan ' num2str(j) ':Scout - Too many files'],'Scout Error','Process Subject','Exclude Subject','Exclude Subject');     
        end
    
        if (strcmp(choice,'Exclude Subject') == 1)
            zephyr_struct.processing_track(i,2) = {'Exclude'};
            save([zephyr_struct.data_directory '/zephyr/zephyr_data.mat'],'zephyr_struct')
        end
        
    end    
    
    
    
    
    
function zephyr_coplanar_test(i,j)

    global zephyr_struct
    
    subject_error = 0;
    path_error = 0;
    file_error = 0;
    
    if (j < 10)
        prefix = '00';
    else
        prefix = '0';
    end		
    
    test_path = ['./raw/' prefix,int2str(j)];

    if (exist(test_path,'dir') ~= 7)
		path_error = 1;
		subject_error = 1;	
        
    else
        [a,b] = unix(['ls ', test_path, ' | wc -l | awk ''{print $1}''']);
        file_count = str2num(b);

        % get number of coplanar slices
        slices = str2num(char(zephyr_struct.study_metadata(j,2)));
        
        if (file_count < (slices + 1))
		    file_error = 1;
			subject_error = 1;
        elseif (file_count > (slices + 1))
            file_error = 2;
			subject_error = 1;
		end
        
	end
    
    if (subject_error == 1)
        
        if (path_error == 1)
            choice = questdlg(['(' char(zephyr_struct.processing_track(i,1)) ') Scan ' num2str(j) ':Coplanar - Path does not exist'],'Coplanar Error','Process Subject','Exclude Subject','Exclude Subject');    
        elseif (file_error == 1)
            choice = questdlg(['(' char(zephyr_struct.processing_track(i,1)) ') Scan ' num2str(j) ':Coplanar - Files are missing'],'Coplanar Error','Process Subject','Exclude Subject','Exclude Subject');
        elseif (file_error == 2)
            choice = questdlg(['(' char(zephyr_struct.processing_track(i,1)) ') Scan ' num2str(j) ':Coplanar - Too many files'],'Coplanar Error','Process Subject','Exclude Subject','Exclude Subject');     
        end
    
        if (strcmp(choice,'Exclude Subject') == 1)
            zephyr_struct.processing_track(i,2) = {'Exclude'};
            save([zephyr_struct.data_directory '/zephyr/zephyr_data.mat'],'zephyr_struct')
        end
        
    end    
    
    
    
    
function zephyr_functional_test(i,j)

    global zephyr_struct
    
    subject_error = 0;
    path_error = 0;
    file_error = 0;
    
    % how many EPI folders should we have?
    % number of slices * number of TRs, divided by 999, and rounded up
    slices = str2num(char(zephyr_struct.study_metadata(j,2)));
	num_images = slices * str2num(char(zephyr_struct.study_metadata(j,5)));
    num_folders_estimate = ceil(num_images / 999);
        
    % determine how many functional folders are present for this scan
    [a,b] = unix(['ls -d ./raw/*', num2str(j), ' | wc -l | awk ''{print $1}''']);
    num_folders_actual = str2num(b);
        
    
    if (num_folders_actual ~= num_folders_estimate)
        path_error = 1;
		subject_error = 1;	
        
    else
        image_count = 0;
            
        % get the number of images present
        for l=0:2:((2*num_folders_estimate)-2) 
            
	        if (l < 10)
                prefix = '0';
            else
                prefix = '';
            end
            
            test_path = ['./raw/' prefix int2str(l) int2str(j)];
            [a,b] = unix(['ls ', test_path, ' | wc -l | awk ''{print $1}''']);
            image_count = image_count + str2num(b) - 1;
        end
         
        tr_count = (image_count / slices) - 1;
        
        % set the file error if need be
        if (tr_count < str2num(char(zephyr_struct.study_metadata(j,5))))
		    file_error = 1;
			subject_error = 1;
        elseif (tr_count > str2num(char(zephyr_struct.study_metadata(j,5))))
            file_error = 2;
			subject_error = 1;
		end
        
    end
    
    if (subject_error == 1)
        
        if (path_error == 1)
            choice = questdlg(['(' char(zephyr_struct.processing_track(i,1)) ') Scan ' num2str(j) ':Functional - A Path does not exist'],'Functional Error','Process Subject','Exclude Subject','Exclude Subject');    
        elseif (file_error == 1)
            choice = questdlg(['(' char(zephyr_struct.processing_track(i,1)) ') Scan ' num2str(j) ':Functional - TRs missing  Estimated number:'],'Functional Error','Process Subject','Exclude Subject','Exclude Subject');
        elseif (file_error == 2)
            choice = questdlg(['(' char(zephyr_struct.processing_track(i,1)) ') Scan ' num2str(j) ':Functional - Too many TRs'],'Functional Error','Process Subject','Exclude Subject','Exclude Subject');     
        end
    
        if (strcmp(choice,'Exclude Subject') == 1)
            zephyr_struct.processing_track(i,2) = {'Exclude'};
            save([zephyr_struct.data_directory '/zephyr/zephyr_data.mat'],'zephyr_struct')
        end
        
    end    
    
    
    
    
    
    
    function zephyr_hires_test(i,j)

    global zephyr_struct
    
    subject_error = 0;
    path_error = 0;
    file_error = 0;
    
    if (j < 10)
        prefix = '00';
    else
        prefix = '0';
    end		
    
    test_path = ['./raw/' prefix,int2str(j)];

    if (exist(test_path,'dir') ~= 7)
		path_error = 1;
		subject_error = 1;	
        
    else
        [a,b] = unix(['ls ', test_path, ' | wc -l | awk ''{print $1}''']);
        file_count = str2num(b);

        % hires is 9 slices
        slices = 124;
        
        if (file_count < (slices + 1))
		    file_error = 1;
			subject_error = 1;
        elseif (file_count > (slices + 1))
            file_error = 2;
			subject_error = 1;
		end
        
	end
    
    if (subject_error == 1)
        
        if (path_error == 1)
            choice = questdlg(['(' char(zephyr_struct.processing_track(i,1)) ') Scan ' num2str(j) ':Hires - Path does not exist'],'Hires Error','Process Subject','Exclude Subject','Exclude Subject');    
        elseif (file_error == 1)
            choice = questdlg(['(' char(zephyr_struct.processing_track(i,1)) ') Scan ' num2str(j) ':Hires - Files are missing'],'Hires Error','Process Subject','Exclude Subject','Exclude Subject');
        elseif (file_error == 2)
            choice = questdlg(['(' char(zephyr_struct.processing_track(i,1)) ') Scan ' num2str(j) ':Hires - Too many files'],'Hires Error','Process Subject','Exclude Subject','Exclude Subject');     
        end
    
        if (strcmp(choice,'Exclude Subject') == 1)
            zephyr_struct.processing_track(i,2) = {'Exclude'};
            save([zephyr_struct.data_directory '/zephyr/zephyr_data.mat'],'zephyr_struct')
        end
        
    end    
    
    
    
    
    
    
    function zephyr_dti_test(i,j)

    global zephyr_struct
    
    subject_error = 0;
    path_error = 0;
    file_error = 0;
    
    if (j < 10)
        prefix = '00';
    else
        prefix = '0';
    end		
    
    test_path = ['./raw/' prefix,int2str(j)];

    if (exist(test_path,'dir') ~= 7)
		path_error = 1;
		subject_error = 1;	
        
    else
        [a,b] = unix(['ls ', test_path, ' | wc -l | awk ''{print $1}''']);
        file_count = str2num(b);

        % dti is 406 slices
        slices = 406;
        
        if (file_count < (slices + 1))
		    file_error = 1;
			subject_error = 1;
        elseif (file_count > (slices + 1))
            file_error = 2;
			subject_error = 1;
		end
        
	end
    
    if (subject_error == 1)
        
        if (path_error == 1)
            choice = questdlg(['(' char(zephyr_struct.processing_track(i,1)) ') Scan ' num2str(j) ':DTI - Path does not exist'],'DTI Error','Process Subject','Exclude Subject','Exclude Subject');    
        elseif (file_error == 1)
            choice = questdlg(['(' char(zephyr_struct.processing_track(i,1)) ') Scan ' num2str(j) ':DTI - Files are missing'],'DTI Error','Process Subject','Exclude Subject','Exclude Subject');
        elseif (file_error == 2)
            choice = questdlg(['(' char(zephyr_struct.processing_track(i,1)) ') Scan ' num2str(j) ':DTI - Too many files'],'DTI Error','Process Subject','Exclude Subject','Exclude Subject');     
        end
    
        if (strcmp(choice,'Exclude Subject') == 1)
            zephyr_struct.processing_track(i,2) = {'Exclude'};
            save([zephyr_struct.data_directory '/zephyr/zephyr_data.mat'],'zephyr_struct')
        end
        
    end    